@@ -79,8 +79,8 @@ int wait_cmd(int fd, struct cdrom_generic_command *cgc, unsigned char *buf,
 	struct request_sense sense;
 	int ret;
 
-	if (cgc->timeout <= 0)
-		cgc->timeout = 500;
+	if (timeout <= 0)
+		timeout = 500;
 
 	memset(&sense, 0, sizeof(sense));
 
@@ -195,13 +195,17 @@ int set_write_mode(int fd, write_params_t *w)
 
 	len = 2 + (((header[0] & 0xff) << 8) | (header[1] & 0xff));
 	offset = 8 + (((header[6] & 0xff) << 8) | (header[7] & 0xff));
-	buffer = calloc(len, sizeof(unsigned char));
-
 	if ((len <= offset+13) || (w->data_block == 10 && len <= offset+51))
 	{
 		perror("mode_sense_write");
-		free(buffer);
-		return ret;
+		return 1;
+	}
+
+	buffer = calloc(len, sizeof(unsigned char));
+	if (!buffer)
+	{
+		perror("calloc");
+		return 1;
 	}
 
 	if ((ret = mode_sense(fd, buffer, GPMODE_WRITE_PARMS_PAGE,
@@ -261,13 +265,17 @@ int get_write_mode(int fd, write_params_t *w)
 
 	len = 2 + (((header[0] & 0xff) << 8) | (header[1] & 0xff));
 	offset = 8 + (((header[6] & 0xff) << 8) | (header[7] & 0xff));
-	buffer = calloc(len, sizeof(unsigned char));
-
 	if (len <= offset+13)
 	{
 		perror("mode_sense_write");
-		free(buffer);
-		return ret;
+		return 1;
+	}
+
+	buffer = calloc(len, sizeof(unsigned char));
+	if (!buffer)
+	{
+		perror("calloc");
+		return 1;
 	}
 
 	if ((ret = mode_sense(fd, buffer, GPMODE_WRITE_PARMS_PAGE,
@@ -351,7 +359,7 @@ int write_file(int fd, struct cdrw_disc *disc)
 
 	while (!ret && go_on) {
 		blocks = disc->fpacket ? disc->packet_size : size / CDROM_BLOCK;
-		ret = read(file, buf, size);
+		ret = read_nointr(file, buf, size);
 		if (ret == -1) {
 			perror("read from file");
 			break;
