@@ -2,7 +2,7 @@
  * libudffs.h
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2014-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2014-2019  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -144,6 +144,9 @@ struct udf_disc
 	uint32_t			*vat;
 	uint64_t			vat_entries;
 
+	short_ad			*metadata_filemap[2];
+	uint32_t			metadata_filemap_count[2];
+
 	struct fileSetDesc		*udf_fsd;
 
 	struct udf_extent		*head;
@@ -193,7 +196,7 @@ struct mbr_partition
 	uint8_t				ending_chs[3];
 	uint32_t			starting_lba;
 	uint32_t			size_in_lba;
-} __attribute__ ((packed));
+} __attribute__ ((packed, may_alias));
 
 #define MBR_BOOT_SIGNATURE		0xAA55
 
@@ -204,7 +207,7 @@ struct mbr
 	uint16_t			unknown;
 	struct mbr_partition		partitions[4];
 	uint16_t			boot_signature;
-} __attribute__ ((packed));
+} __attribute__ ((packed, may_alias));
 
 /* crc.c */
 extern uint16_t udf_crc(uint8_t *, uint32_t, uint16_t);
@@ -237,5 +240,7 @@ size_t gen_uuid_from_vol_set_ident(char[17], const dstring *, size_t);
 uint32_t strtou32(const char *, int, int *);
 uint16_t strtou16(const char *, int, int *);
 uint32_t randu32(void);
+ssize_t read_nointr(int, void *, size_t);
+ssize_t write_nointr(int, const void *, size_t);
 
 #endif /* __LIBUDFFS_H */
