@@ -2,7 +2,7 @@
  * unicode.c
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2014-2017  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2014-2019  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,12 +30,14 @@
 
 #include "libudffs.h"
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <wchar.h>
 #include <errno.h>
+#include <limits.h>
 
 size_t decode_utf8(const dchars *in, char *out, size_t inlen, size_t outlen)
 {
@@ -72,6 +74,17 @@ size_t decode_utf8(const dchars *in, char *out, size_t inlen, size_t outlen)
 			out[len++] = (uint8_t)(0xc0 | (c >> 6));
 			out[len++] = (uint8_t)(0x80 | (c & 0x3f));
 		}
+		else if (c >= 0xD800 && c <= 0xDBFF && i+1 < inlen && (((unsigned int)in[i] << 8) | in[i+1]) >= 0xDC00 && (((unsigned int)in[i] << 8) | in[i+1]) <= 0xDFFF)
+		{
+			c = 0x10000 + ((c - 0xD800) << 10) + ((((unsigned int)in[i] << 8) | in[i+1]) - 0xDC00);
+			i += 2;
+			if (len+4 >= outlen)
+				return (size_t)-1;
+			out[len++] = (uint8_t)(0xf0 | (c >> 18));
+			out[len++] = (uint8_t)(0x80 | ((c >> 12) & 0x3f));
+			out[len++] = (uint8_t)(0x80 | ((c >> 6) & 0x3f));
+			out[len++] = (uint8_t)(0x80 | (c & 0x3f));
+		}
 		else
 		{
 			if (len+3 >= outlen)
@@ -136,16 +149,6 @@ try_again:
 					utf_char = c & 0x07;
 					utf_cnt = 3;
 				}
-				else if ((c & 0xFC) == 0xF8)
-				{
-					utf_char = c & 0x03;
-					utf_cnt = 4;
-				}
-				else if ((c & 0xFE) == 0xfc)
-				{
-					utf_char = c & 0x01;
-					utf_cnt = 5;
-				}
 				else
 					goto error_invalid;
 				continue;
@@ -163,14 +166,14 @@ try_again:
 			if (max_val == 0x7F)
 			{
 				len = 1;
-				max_val = 0xFFFF;
+				max_val = 0x10FFFF;
 				out[0] = 0x10;
 				goto try_again;
 			}
 			return (size_t)-1;
 		}
 
-		if (max_val == 0xFFFF)
+		if (max_val == 0x10FFFF)
 		{
 			if (len + 2 > outlen)
 			{
@@ -179,6 +182,14 @@ try_again:
 				out[0] = 0x8;
 				goto try_again;
 			}
+			if (utf_char > 0xFFFF)
+			{
+				if (len+4 > outlen)
+					return (size_t)-1;
+				out[len++] = ((((utf_char - 0x10000) >> 10) + 0xD800) >> 8) & 0xFF;
+				out[len++] = (((utf_char - 0x10000) >> 10) + 0xD800) & 0xFF;
+				utf_char = ((utf_char - 0x10000) & 0x3FF) + 0xDC00;
+			}
 			out[len++] = utf_char >> 8;
 		}
 		if (len + 1 > outlen)
@@ -202,7 +213,7 @@ size_t decode_locale(const dchars *in, char *out, size_t inlen, size_t outlen)
 	size_t wcslen, clen;
 	wchar_t *wcs;
 	mbstate_t ps;
-	char cbuf[MB_CUR_MAX];
+	char cbuf[MB_LEN_MAX];
 
 	if (outlen == 0)
 		return (size_t)-1;
@@ -225,7 +236,20 @@ size_t decode_locale(const dchars *in, char *out, size_t inlen, size_t outlen)
 	{
 		wcs[len] = in[i++];
 		if (in[0] == 16)
+		{
 			wcs[len] = (wcs[len] << 8) | in[i++];
+#if WCHAR_MAX >= 0x10FFFF
+			if (wcs[len] >= 0xD800 && wcs[len] <= 0xDBFF && i+1 < inlen)
+			{
+				wchar_t ch = (((wchar_t)in[i] << 8) | in[i+1]);
+				if (ch >= 0xDC00 && ch <= 0xDFFF)
+				{
+					wcs[len] = 0x10000 + ((wcs[len] - 0xD800) << 10) + (ch - 0xDC00);
+					i += 2;
+				}
+			}
+#endif
+		}
 		++len;
 	}
 
@@ -263,6 +287,12 @@ size_t decode_locale(const dchars *in, char *out, size_t inlen, size_t outlen)
 	return len;
 }
 
+#if WCHAR_MAX > 0x10FFFF
+#define DCHAR_IN_WCHAR_MAX 0x10FFFF
+#else
+#define DCHAR_IN_WCHAR_MAX WCHAR_MAX
+#endif
+
 size_t encode_locale(dchars *out, const char *in, size_t outlen)
 {
 	size_t i;
@@ -298,13 +328,13 @@ try_again:
 			{
 				len = 1;
 				out[0] = 16;
-				max_val = 0xFFFF;
+				max_val = DCHAR_IN_WCHAR_MAX;
 				goto try_again;
 			}
 			goto error_out;
 		}
 
-		if (max_val == 0xFFFF)
+		if (max_val == DCHAR_IN_WCHAR_MAX)
 		{
 			if (len+2 > outlen)
 			{
@@ -313,6 +343,16 @@ try_again:
 				max_val = 0xFF;
 				goto try_again;
 			}
+#if WCHAR_MAX > 0xFFFF
+			if (wcs[i] > 0xFFFF)
+			{
+				if (len+4 > outlen)
+					goto error_out;
+				out[len++] = ((((wcs[i] - 0x10000) >> 10) + 0xD800) >> 8) & 0xFF;
+				out[len++] = (((wcs[i] - 0x10000) >> 10) + 0xD800) & 0xFF;
+				wcs[i] = ((wcs[i] - 0x10000) & 0x3FF) + 0xDC00;
+			}
+#endif
 			out[len++] = (wcs[i] >> 8) & 0xFF;
 		}
 
