@@ -2,7 +2,7 @@
  * file.c
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2016-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2016-2019  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -104,7 +105,7 @@ tag query_tag(struct udf_disc *disc, struct udf_extent *ext, struct udf_desc *de
 	data = desc->data;
 	while (data != NULL)
 	{
-		crc = udf_crc(data->buffer + offset, data->length - offset, crc);
+		crc = udf_crc((uint8_t *)data->buffer + offset, data->length - offset, crc);
 		offset = 0;
 		data = data->next;
 	}
@@ -152,7 +153,7 @@ tag udf_query_tag(struct udf_disc *disc, uint16_t Ident, uint16_t SerialNum, uin
 	{
 		if ((clength = data->length) > length)
 			clength = length;
-		crc = udf_crc(data->buffer + skip + offset, clength - offset, crc);
+		crc = udf_crc((uint8_t *)data->buffer + skip + offset, clength - offset, crc);
 		length -= clength;
 		offset = 0;
 		skip = 0;
@@ -206,8 +207,13 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 					parent->length += sizeof(short_ad);
 					parent->data->length += sizeof(short_ad);
 					parent->data->buffer = realloc(parent->data->buffer, parent->length);
+					if (!parent->data->buffer)
+					{
+						fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+						exit(1);
+					}
 					efe = (struct extendedFileEntry *)parent->data->buffer;
-					sad = (short_ad *)&efe->allocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs)];
+					sad = (short_ad *)&efe->extendedAttrAndAllocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs)];
 					sad->extPosition = cpu_to_le32(block);
 					sad->extLength = cpu_to_le32(data->length);
 					efe->lengthAllocDescs = cpu_to_le32(sizeof(short_ad));
@@ -219,8 +225,13 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 					parent->length += sizeof(long_ad);
 					parent->data->length += sizeof(long_ad);
 					parent->data->buffer = realloc(parent->data->buffer, parent->length);
+					if (!parent->data->buffer)
+					{
+						fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+						exit(1);
+					}
 					efe = (struct extendedFileEntry *)parent->data->buffer;
-					lad = (long_ad *)&efe->allocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs)];
+					lad = (long_ad *)&efe->extendedAttrAndAllocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs)];
 					lad->extLocation.logicalBlockNum = cpu_to_le32(block);
 					lad->extLocation.partitionReferenceNum = cpu_to_le16(0);
 					lad->extLength = cpu_to_le32(data->length);
@@ -234,7 +245,7 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 				{
 					short_ad *sad;
 
-					sad = (short_ad *)&efe->allocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs) - sizeof(short_ad)];
+					sad = (short_ad *)&efe->extendedAttrAndAllocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs) - sizeof(short_ad)];
 					fiddesc = find_desc(pspace, le32_to_cpu(sad->extPosition));
 					block = fiddesc->offset;
 					append_data(fiddesc, data);
@@ -244,7 +255,7 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 				{
 					long_ad *lad;
 
-					lad = (long_ad *)&efe->allocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs) - sizeof(long_ad)];
+					lad = (long_ad *)&efe->extendedAttrAndAllocDescs[le32_to_cpu(efe->lengthExtendedAttr) + le32_to_cpu(efe->lengthAllocDescs) - sizeof(long_ad)];
 					fiddesc = find_desc(pspace, le32_to_cpu(lad->extLocation.logicalBlockNum));
 					block = fiddesc->offset;
 					append_data(fiddesc, data);
@@ -278,8 +289,13 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 					parent->length += sizeof(short_ad);
 					parent->data->length += sizeof(short_ad);
 					parent->data->buffer = realloc(parent->data->buffer, parent->length);
+					if (!parent->data->buffer)
+					{
+						fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+						exit(1);
+					}
 					fe = (struct fileEntry *)parent->data->buffer;
-					sad = (short_ad *)&fe->allocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs)];
+					sad = (short_ad *)&fe->extendedAttrAndAllocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs)];
 					sad->extPosition = cpu_to_le32(block);
 					sad->extLength = cpu_to_le32(data->length);
 					fe->lengthAllocDescs = cpu_to_le32(sizeof(short_ad));
@@ -291,8 +307,13 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 					parent->length += sizeof(long_ad);
 					parent->data->length += sizeof(long_ad);
 					parent->data->buffer = realloc(parent->data->buffer, parent->length);
+					if (!parent->data->buffer)
+					{
+						fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+						exit(1);
+					}
 					fe = (struct fileEntry *)parent->data->buffer;
-					lad = (long_ad *)&fe->allocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs)];
+					lad = (long_ad *)&fe->extendedAttrAndAllocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs)];
 					lad->extLocation.logicalBlockNum = cpu_to_le32(block);
 					lad->extLocation.partitionReferenceNum = cpu_to_le16(0);
 					lad->extLength = cpu_to_le32(data->length);
@@ -306,7 +327,7 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 				{
 					short_ad *sad;
 
-					sad = (short_ad *)&fe->allocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs) - sizeof(short_ad)];
+					sad = (short_ad *)&fe->extendedAttrAndAllocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs) - sizeof(short_ad)];
 					fiddesc = find_desc(pspace, le32_to_cpu(sad->extPosition));
 					block = fiddesc->offset;
 					append_data(fiddesc, data);
@@ -316,7 +337,7 @@ int insert_desc(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 				{
 					long_ad *lad;
 
-					lad = (long_ad *)&fe->allocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs) - sizeof(long_ad)];
+					lad = (long_ad *)&fe->extendedAttrAndAllocDescs[le32_to_cpu(fe->lengthExtendedAttr) + le32_to_cpu(fe->lengthAllocDescs) - sizeof(long_ad)];
 					fiddesc = find_desc(pspace, le32_to_cpu(lad->extLocation.logicalBlockNum));
 					block = fiddesc->offset;
 					append_data(fiddesc, data);
@@ -441,7 +462,7 @@ void insert_fid(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 		fid->fileCharacteristics = filechar;
 		fid->lengthFileIdent = length;
 		fid->lengthOfImpUse = cpu_to_le16(0);
-		memcpy(fid->fileIdent, name, length);
+		memcpy(fid->impUseAndFileIdent, name, length);
 		fid->descTag = udf_query_tag(disc, TAG_IDENT_FID, 1, le32_to_cpu(fid->descTag.tagLocation), data, 0, ilength);
 
 		efe->informationLength = cpu_to_le64(le64_to_cpu(efe->informationLength) + ilength);
@@ -474,7 +495,7 @@ void insert_fid(struct udf_disc *disc, struct udf_extent *pspace, struct udf_des
 		fid->fileCharacteristics = filechar;
 		fid->lengthFileIdent = length;
 		fid->lengthOfImpUse = cpu_to_le16(0);
-		memcpy(fid->fileIdent, name, length);
+		memcpy(fid->impUseAndFileIdent, name, length);
 		fid->descTag = udf_query_tag(disc, TAG_IDENT_FID, 1, le32_to_cpu(fid->descTag.tagLocation), data, 0, ilength);
 		fe->informationLength = cpu_to_le64(le64_to_cpu(fe->informationLength) + ilength);
 	}
@@ -497,13 +518,13 @@ void insert_ea(struct udf_disc *disc, struct udf_desc *desc, struct genericForma
 		{                                                             \
 			efe = (struct extendedFileEntry *)desc->data->buffer; \
 			lengthExtendedAttr = efe->lengthExtendedAttr;         \
-			extendedAttr = efe->extendedAttr;                     \
+			extendedAttr = efe->extendedAttrAndAllocDescs;        \
 		}                                                             \
 		else                                                          \
 		{                                                             \
 			fe = (struct fileEntry *)desc->data->buffer;          \
 			lengthExtendedAttr = fe->lengthExtendedAttr;          \
-			extendedAttr = fe->extendedAttr;                      \
+			extendedAttr = fe->extendedAttrAndAllocDescs;         \
 		}                                                             \
 		ea_hdr = (struct extendedAttrHeaderDesc *)extendedAttr;       \
 	} while ( 0 )
@@ -521,19 +542,25 @@ void insert_ea(struct udf_disc *disc, struct udf_desc *desc, struct genericForma
 		desc->data->length += sizeof(*ea_hdr);
 		desc->data->buffer = realloc(desc->data->buffer, desc->data->length);
 
+		if (!desc->data->buffer)
+		{
+			fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
+
 		UPDATE_PTR;
 
 		if (disc->flags & FLAG_EFE)
 		{
 			lengthExtendedAttr = efe->lengthExtendedAttr = cpu_to_le32(sizeof(*ea_hdr));
 			if (le32_to_cpu(efe->lengthAllocDescs))
-				memmove(&efe->allocDescs[sizeof(*ea_hdr)], efe->allocDescs, le32_to_cpu(efe->lengthAllocDescs));
+				memmove(&efe->extendedAttrAndAllocDescs[sizeof(*ea_hdr)], efe->extendedAttrAndAllocDescs, le32_to_cpu(efe->lengthAllocDescs));
 		}
 		else
 		{
 			lengthExtendedAttr = fe->lengthExtendedAttr = cpu_to_le32(sizeof(*ea_hdr));
 			if (le32_to_cpu(fe->lengthAllocDescs))
-				memmove(&fe->allocDescs[sizeof(*ea_hdr)], fe->allocDescs, le32_to_cpu(fe->lengthAllocDescs));
+				memmove(&fe->extendedAttrAndAllocDescs[sizeof(*ea_hdr)], fe->extendedAttrAndAllocDescs, le32_to_cpu(fe->lengthAllocDescs));
 		}
 
 		ea_hdr->impAttrLocation = cpu_to_le32(0xFFFFFFFF);
@@ -546,6 +573,12 @@ void insert_ea(struct udf_disc *disc, struct udf_desc *desc, struct genericForma
 		desc->data->length += length;
 		desc->data->buffer = realloc(desc->data->buffer, desc->data->length);
 
+		if (!desc->data->buffer)
+		{
+			fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
+
 		UPDATE_PTR;
 
 		if (le32_to_cpu(ea_hdr->appAttrLocation) != 0xFFFFFFFF)
@@ -582,6 +615,12 @@ void insert_ea(struct udf_disc *disc, struct udf_desc *desc, struct genericForma
 		desc->data->length += length;
 		desc->data->buffer = realloc(desc->data->buffer, desc->data->length);
 
+		if (!desc->data->buffer)
+		{
+			fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
+
 		UPDATE_PTR;
 
 		if (le32_to_cpu(ea_hdr->appAttrLocation) != 0xFFFFFFFF)
@@ -616,6 +655,12 @@ void insert_ea(struct udf_disc *disc, struct udf_desc *desc, struct genericForma
 		desc->data->length += length;
 		desc->data->buffer = realloc(desc->data->buffer, desc->data->length);
 
+		if (!desc->data->buffer)
+		{
+			fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
+
 		UPDATE_PTR;
 
 		if (le32_to_cpu(ea_hdr->appAttrLocation) == 0xFFFFFFFF)
@@ -866,7 +911,14 @@ static inline unsigned long udf_find_next_one_bit (void * addr, unsigned long si
 	offset &= (BITS_PER_LONG-1);
 	if (offset)
 	{
-		tmp = leBPL_to_cpup(p++);
+		if (size >= BITS_PER_LONG)
+			tmp = leBPL_to_cpup(p++);
+		else
+		{
+			tmp = 0;
+			memcpy(&tmp, p, (size+7)/8);
+			tmp = leBPL_to_cpup(&tmp);
+		}
 		tmp &= ~0UL << offset;
 		if (size < BITS_PER_LONG)
 			goto found_first;
@@ -913,7 +965,14 @@ static inline unsigned long udf_find_next_zero_bit(void * addr, unsigned long si
 	offset &= (BITS_PER_LONG-1);
 	if (offset)
 	{
-		tmp = leBPL_to_cpup(p++);
+		if (size >= BITS_PER_LONG)
+			tmp = leBPL_to_cpup(p++);
+		else
+		{
+			tmp = 0;
+			memcpy(&tmp, p, (size+7)/8);
+			tmp = leBPL_to_cpup(&tmp);
+		}
 		tmp |= (~0UL >> (BITS_PER_LONG-offset));
 		if (size < BITS_PER_LONG)
 			goto found_first;
@@ -1052,8 +1111,11 @@ int udf_alloc_blocks(struct udf_disc *disc, struct udf_extent *pspace, uint32_t
 {
 	struct udf_desc *desc;
 	struct partitionHeaderDesc *phd = (struct partitionHeaderDesc *)disc->udf_pd[0]->partitionContentsUse;
+	uint32_t value;
 
-	disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - blocks);
+	memcpy(&value, &disc->udf_lvid->data[sizeof(uint32_t)*0], sizeof(value));
+	value = cpu_to_le32(le32_to_cpu(value) - blocks);
+	memcpy(&disc->udf_lvid->data[sizeof(uint32_t)*0], &value, sizeof(value));
 
 	if (disc->flags & FLAG_FREED_BITMAP)
 	{
@@ -1090,6 +1152,11 @@ int udf_alloc_blocks(struct udf_disc *disc, struct udf_extent *pspace, uint32_t
 			fprintf(stderr, "%s: Error: Not enough blocks on device\n", appname);
 			exit(1);
 		}
+		if ((disc->vat_entries + blocks) * sizeof(uint32_t) > disc->blocksize)
+		{
+			fprintf(stderr, "%s: Error: Too many blocks for Virtual Allocation Table\n", appname);
+			exit(1);
+		}
 		for (i = 0; i < blocks; ++i)
 			disc->vat[disc->vat_entries++] = cpu_to_le32(start+i);
 		return start;
