@@ -2,7 +2,7 @@
  * mkudffs.c
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2014-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2014-2019  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <inttypes.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -57,16 +58,15 @@ void udf_init_disc(struct udf_disc *disc)
 	memset(disc, 0x00, sizeof(*disc));
 
 	disc->blocksize = 2048;
-	disc->udf_rev = le16_to_cpu(default_lvidiu.minUDFReadRev);
-	disc->flags = FLAG_LOCALE | FLAG_CLOSED | FLAG_EFE;
+	disc->flags = FLAG_LOCALE;
 	disc->blkssz = 512;
 	disc->mode = 0755;
 
 	if (gettimeofday(&tv, NULL) != 0 || tv.tv_sec == (time_t)-1 || (tm = localtime(&tv.tv_sec)) == NULL || tm->tm_year < 1-1900 || tm->tm_year > 9999-1900)
 	{
-		/* fallback to 1.1.1980 00:00:00 */
+		/* fallback to 1970-01-01 00:00:00 */
 		ts.typeAndTimezone = cpu_to_le16(0x1000);
-		ts.year = cpu_to_le16(1980);
+		ts.year = cpu_to_le16(1970);
 		ts.month = 1;
 		ts.day = 1;
 		ts.hour = 0;
@@ -102,24 +102,49 @@ void udf_init_disc(struct udf_disc *disc)
 
 	/* Allocate/Initialize Descriptors */
 	disc->udf_pvd[0] = malloc(sizeof(struct primaryVolDesc));
+	if (!disc->udf_pvd[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_pvd[0], &default_pvd, sizeof(struct primaryVolDesc));
 	memcpy(&disc->udf_pvd[0]->recordingDateAndTime, &ts, sizeof(timestamp));
-	snprintf(uuid, sizeof(uuid), "%08lx%08lx", (unsigned long int)uuid_time, (unsigned long int)randu32());
+	snprintf(uuid, sizeof(uuid), "%08" PRIx32 "%08" PRIx32, uuid_time, randu32());
 	memcpy(&disc->udf_pvd[0]->volSetIdent[1], uuid, 16);
 	disc->udf_pvd[0]->volIdent[31] = strlen((char *)disc->udf_pvd[0]->volIdent);
 	disc->udf_pvd[0]->volSetIdent[127] = strlen((char *)disc->udf_pvd[0]->volSetIdent);
 
 	disc->udf_lvd[0] = malloc(sizeof(struct logicalVolDesc));
+	if (!disc->udf_lvd[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_lvd[0], &default_lvd, sizeof(struct logicalVolDesc));
 	disc->udf_lvd[0]->logicalVolIdent[127] = strlen((char *)disc->udf_lvd[0]->logicalVolIdent);
 
 	disc->udf_pd[0] = malloc(sizeof(struct partitionDesc));
+	if (!disc->udf_pd[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_pd[0], &default_pd, sizeof(struct partitionDesc));
 
 	disc->udf_usd[0] = malloc(sizeof(struct unallocSpaceDesc));
+	if (!disc->udf_usd[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_usd[0], &default_usd, sizeof(struct unallocSpaceDesc));
 
 	disc->udf_iuvd[0] = malloc(sizeof(struct impUseVolDesc) + sizeof(struct impUseVolDescImpUse));
+	if (!disc->udf_iuvd[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_iuvd[0], &default_iuvd, sizeof(struct impUseVolDesc));
 	memcpy(query_iuvdiu(disc), &default_iuvdiu, sizeof(struct impUseVolDescImpUse));
 	query_iuvdiu(disc)->logicalVolIdent[127] = strlen((char *)query_iuvdiu(disc)->logicalVolIdent);
@@ -128,20 +153,37 @@ void udf_init_disc(struct udf_disc *disc)
 	query_iuvdiu(disc)->LVInfo3[35] = strlen((char *)query_iuvdiu(disc)->LVInfo3);
 
 	disc->udf_td[0] = malloc(sizeof(struct terminatingDesc));
+	if (!disc->udf_td[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_td[0], &default_td, sizeof(struct terminatingDesc));
 
 	disc->udf_lvid = malloc(sizeof(struct logicalVolIntegrityDesc) + sizeof(struct logicalVolIntegrityDescImpUse));
+	if (!disc->udf_lvid)
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_lvid, &default_lvid, sizeof(struct logicalVolIntegrityDesc));
 	memcpy(&disc->udf_lvid->recordingDateAndTime, &ts, sizeof(timestamp));
 	memcpy(query_lvidiu(disc), &default_lvidiu, sizeof(struct logicalVolIntegrityDescImpUse));
 
 	disc->udf_stable[0] = malloc(sizeof(struct sparingTable));
+	if (!disc->udf_stable[0])
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_stable[0], &default_stable, sizeof(struct sparingTable));
 
-	disc->vat = calloc(1, disc->blocksize);
-	disc->vat_entries = 0;
-
 	disc->udf_fsd = malloc(sizeof(struct fileSetDesc));
+	if (!disc->udf_fsd)
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	memcpy(disc->udf_fsd, &default_fsd, sizeof(struct fileSetDesc));
 	memcpy(&disc->udf_fsd->recordingDateAndTime, &ts, sizeof(timestamp));
 	disc->udf_fsd->logicalVolIdent[127] = strlen((char *)disc->udf_fsd->logicalVolIdent);
@@ -150,6 +192,11 @@ void udf_init_disc(struct udf_disc *disc)
 	disc->udf_fsd->abstractFileIdent[31] = strlen((char *)disc->udf_fsd->abstractFileIdent);
 
 	disc->head = malloc(sizeof(struct udf_extent));
+	if (!disc->head)
+	{
+		fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	disc->tail = disc->head;
 
 	disc->head->space_type = USPACE;
@@ -159,12 +206,15 @@ void udf_init_disc(struct udf_disc *disc)
 	disc->head->prev = NULL;
 	disc->head->head = NULL;
 	disc->head->tail = NULL;
+
+	udf_set_version(disc, 0x0201);
 }
 
 int udf_set_version(struct udf_disc *disc, uint16_t udf_rev)
 {
 	struct logicalVolIntegrityDescImpUse *lvidiu;
-	uint16_t udf_rev_le16;
+	struct domainIdentSuffix *dis;
+	struct UDFIdentSuffix *uis;
 
 	if (disc->udf_rev == udf_rev)
 		return 0;
@@ -182,29 +232,50 @@ int udf_set_version(struct udf_disc *disc, uint16_t udf_rev)
 	else
 		disc->udf_rev = udf_rev;
 
-	if (disc->udf_rev >= 0x0200)
-
+	if (udf_rev >= 0x0200)
 	{
 		disc->flags |= FLAG_EFE;
 		strcpy((char *)disc->udf_pd[0]->partitionContents.ident, PD_PARTITION_CONTENTS_NSR03);
 	}
-	else if (disc->udf_rev == 0x0150)
+	else
 	{
 		disc->flags &= ~FLAG_EFE;
 		strcpy((char *)disc->udf_pd[0]->partitionContents.ident, PD_PARTITION_CONTENTS_NSR02);
 	}
-	else // 0x0102 and older
-	{
-		disc->flags &= ~FLAG_VAT;
-		disc->flags &= ~FLAG_EFE;
-		strcpy((char *)disc->udf_pd[0]->partitionContents.ident, PD_PARTITION_CONTENTS_NSR02);
-	}
 
-	udf_rev_le16 = cpu_to_le16(udf_rev);
-	memcpy(disc->udf_fsd->domainIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
-	memcpy(disc->udf_lvd[0]->domainIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
-	memcpy(disc->udf_iuvd[0]->impIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
-	memcpy(disc->udf_stable[0]->sparingIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
+	dis = (struct domainIdentSuffix *)disc->udf_fsd->domainIdent.identSuffix;
+	dis->UDFRevision = cpu_to_le16(udf_rev);
+	dis = (struct domainIdentSuffix *)disc->udf_lvd[0]->domainIdent.identSuffix;
+	dis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)disc->udf_iuvd[0]->impIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)disc->udf_stable[0]->sparingIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+
+	dis = (struct domainIdentSuffix *)default_lvd.domainIdent.identSuffix;
+	dis->UDFRevision = cpu_to_le16(udf_rev);
+	dis = (struct domainIdentSuffix *)default_fsd.domainIdent.identSuffix;
+	dis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)default_iuvd.impIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)default_stable.sparingIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)default_iuea.impIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)default_vat15.vatIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)default_sparmap.partIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+	uis = (struct UDFIdentSuffix *)default_virtmap.partIdent.identSuffix;
+	uis->UDFRevision = cpu_to_le16(udf_rev);
+
+	if (udf_rev >= 0x0200)
+	{
+		default_vat20.minUDFReadRev = cpu_to_le16(udf_rev);
+		default_vat20.minUDFWriteRev = cpu_to_le16(udf_rev);
+		default_vat20.maxUDFWriteRev = cpu_to_le16(udf_rev);
+	}
+
 	lvidiu = query_lvidiu(disc);
 	if (udf_rev < 0x0102)
 	{
@@ -213,6 +284,10 @@ int udf_set_version(struct udf_disc *disc, uint16_t udf_rev)
 		lvidiu->minUDFReadRev = cpu_to_le16(0);
 		lvidiu->minUDFWriteRev = cpu_to_le16(0);
 		lvidiu->maxUDFWriteRev = cpu_to_le16(0);
+
+		default_lvidiu.minUDFReadRev = cpu_to_le16(0);
+		default_lvidiu.minUDFWriteRev = cpu_to_le16(0);
+		default_lvidiu.maxUDFWriteRev = cpu_to_le16(0);
 	}
 	else
 	{
@@ -222,6 +297,13 @@ int udf_set_version(struct udf_disc *disc, uint16_t udf_rev)
 			lvidiu->minUDFReadRev = cpu_to_le16(udf_rev);
 		lvidiu->minUDFWriteRev = cpu_to_le16(udf_rev);
 		lvidiu->maxUDFWriteRev = cpu_to_le16(udf_rev);
+
+		if (udf_rev == 0x0260)
+			default_lvidiu.minUDFReadRev = cpu_to_le16(0x0250);
+		else
+			default_lvidiu.minUDFReadRev = cpu_to_le16(udf_rev);
+		default_lvidiu.minUDFWriteRev = cpu_to_le16(udf_rev);
+		default_lvidiu.maxUDFWriteRev = cpu_to_le16(udf_rev);
 	}
 	return 0;
 }
@@ -235,7 +317,7 @@ void split_space(struct udf_disc *disc)
 	struct sparablePartitionMap *spm;
 	struct udf_extent *ext;
 	uint32_t accessType;
-	uint32_t i, j;
+	uint32_t i, value;
 
 	// OS boot area
 	if (disc->flags & FLAG_BOOTAREA_MBR)
@@ -253,22 +335,19 @@ void split_space(struct udf_disc *disc)
 	if (blocks > 257)
 		set_extent(disc, ANCHOR, 256, 1);
 
-	// Second anchor point at sector (End-Of-Volume - 256)
-	if (disc->flags & FLAG_CLOSED)
+	// Second anchor point at sector (End-Of-Volume - 256) or 257 for VAT
+	if ((disc->flags & FLAG_VAT) && (disc->flags & FLAG_CLOSED))
 	{
-		if (disc->flags & FLAG_VAT)
+		if (blocks <= 257 || blocks-257 <= 256)
 		{
-			if (blocks <= 257 || blocks-257 <= 256)
-			{
-				fprintf(stderr, "%s: Error: Not enough blocks on device\n", appname);
-				exit(1);
-			}
-			set_extent(disc, ANCHOR, 257, 1);
-		}
-		else if (blocks >= 3072)
-		{
-			set_extent(disc, ANCHOR, blocks-257, 1);
+			fprintf(stderr, "%s: Error: Not enough blocks on device\n", appname);
+			exit(1);
 		}
+		set_extent(disc, ANCHOR, 257, 1);
+	}
+	else if (blocks >= 3072)
+	{
+		set_extent(disc, ANCHOR, blocks-257, 1);
 	}
 
 	// Final anchor point at sector End-Of-Volume/Session for sequentially writable media
@@ -402,6 +481,11 @@ void split_space(struct udf_disc *disc)
 				size = ext->blocks;
 			}
 
+			// Make sure partition doesn't have a space bitmap with a partial last byte.
+			// It's a corner case that some systems (Windows chkdsk) don't handle properly.
+			if ((disc->flags & FLAG_UNALLOC_BITMAP) && (size & 7))
+				size &= ~7;
+
 			// round size down to a multiple of alignment/packet_size
 			if (size % offsets[PSPACE_SIZE])
 				size -= (size % offsets[PSPACE_SIZE]);
@@ -430,16 +514,18 @@ void split_space(struct udf_disc *disc)
 	for (i=0; i<le32_to_cpu(disc->udf_lvd[0]->numPartitionMaps); i++)
 	{
 		if (i == 1)
-			disc->udf_lvid->freeSpaceTable[i] = cpu_to_le32(0xFFFFFFFF);
+			value = cpu_to_le32(0xFFFFFFFF);
 		else
-			disc->udf_lvid->freeSpaceTable[i] = cpu_to_le32(size);
+			value = cpu_to_le32(size);
+		memcpy(&disc->udf_lvid->data[sizeof(uint32_t)*i], &value, sizeof(value));
 	}
-	for (j=0; j<le32_to_cpu(disc->udf_lvd[0]->numPartitionMaps); j++)
+	for (i=0; i<le32_to_cpu(disc->udf_lvd[0]->numPartitionMaps); i++)
 	{
-		if (j == 1)
-			disc->udf_lvid->sizeTable[i+j] = cpu_to_le32(0xFFFFFFFF);
+		if (i == 1)
+			value = cpu_to_le32(0xFFFFFFFF);
 		else
-			disc->udf_lvid->sizeTable[i+j] = cpu_to_le32(size);
+			value = cpu_to_le32(size);
+		memcpy(&disc->udf_lvid->data[sizeof(uint32_t)*le32_to_cpu(disc->udf_lvd[0]->numPartitionMaps) + sizeof(uint32_t)*i], &value, sizeof(value));
 	}
 
 	if (!next_extent(disc->head, LVID))
@@ -463,7 +549,7 @@ void dump_space(struct udf_disc *disc)
 
 	while (start_ext != NULL)
 	{
-		printf("start=%lu, blocks=%lu, type=", (unsigned long int)start_ext->start, (unsigned long int)start_ext->blocks);
+		printf("start=%"PRIu32", blocks=%"PRIu32", type=", start_ext->start, start_ext->blocks);
 		for (i=0; i<UDF_SPACE_TYPE_SIZE; i++)
 		{
 			if (!(start_ext->space_type & (1<<i)))
@@ -509,15 +595,15 @@ static void fill_mbr(struct udf_disc *disc, struct mbr *mbr, uint32_t start)
 
 	if (fd >= 0 && lseek(fd, ((off_t)start) * disc->blocksize, SEEK_SET) >= 0)
 	{
-		if (read(fd, &old_mbr, sizeof(struct mbr)) == sizeof(struct mbr))
+		if (read_nointr(fd, &old_mbr, sizeof(struct mbr)) == sizeof(struct mbr))
 		{
-			if (old_mbr.boot_signature == constant_cpu_to_le16(MBR_BOOT_SIGNATURE))
-				mbr->disk_signature = le32_to_cpu(old_mbr.disk_signature);
+			if (le16_to_cpu(old_mbr.boot_signature) == MBR_BOOT_SIGNATURE)
+				mbr->disk_signature = old_mbr.disk_signature;
 		}
 	}
 
 	if (!mbr->disk_signature)
-		mbr->disk_signature = le32_to_cpu(randu32());
+		mbr->disk_signature = cpu_to_le32(randu32());
 
 	lba_blocks = ((uint64_t)disc->blocks * disc->blocksize + disc->blkssz - 1) / disc->blkssz;
 
@@ -648,12 +734,22 @@ void setup_anchor(struct udf_disc *disc)
 	do
 	{
 		ext->head = ext->tail = malloc(sizeof(struct udf_desc) + sizeof(struct udf_data));
-		ext->head->data = (struct udf_data *)&(ext->head)[1];
+		if (!ext->head)
+		{
+			fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
+		ext->head->data = (struct udf_data *)((uint8_t *)ext->head + sizeof(struct udf_desc));
 		ext->head->data->next = ext->head->data->prev = NULL;
 		ext->head->ident = TAG_IDENT_AVDP;
 		ext->head->offset = 0;
 		ext->head->length = ext->head->data->length = sizeof(struct anchorVolDescPtr);
 		disc->udf_anchor[i] = ext->head->data->buffer = malloc(sizeof(struct anchorVolDescPtr));
+		if (!disc->udf_anchor[i])
+		{
+			fprintf(stderr, "%s: Error: malloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
 		ext->head->next = ext->head->prev = NULL;
 		disc->udf_anchor[i]->mainVolDescSeqExt.extLocation = cpu_to_le32(mloc);
 		disc->udf_anchor[i]->mainVolDescSeqExt.extLength = cpu_to_le32(mlen);
@@ -669,6 +765,17 @@ void setup_partition(struct udf_disc *disc)
 {
 	struct udf_extent *pspace;
 
+	if (disc->flags & FLAG_VAT)
+	{
+		disc->vat = calloc(1, disc->blocksize);
+		if (!disc->vat)
+		{
+			fprintf(stderr, "%s: Error: calloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
+		disc->vat_entries = 0;
+	}
+
 	pspace = next_extent(disc->head, PSPACE);
 	if (!pspace)
 	{
@@ -687,12 +794,15 @@ int setup_space(struct udf_disc *disc, struct udf_extent *pspace, uint32_t offse
 	struct udf_desc *desc;
 	struct partitionHeaderDesc *phd = (struct partitionHeaderDesc *)disc->udf_pd[0]->partitionContentsUse;
 	uint32_t length = (sizeof(struct spaceBitmapDesc) + (pspace->blocks+7)/8 + disc->blocksize-1) / disc->blocksize * disc->blocksize;
+	uint32_t value;
+
+	memcpy(&value, &disc->udf_lvid->data[sizeof(uint32_t)*0], sizeof(value));
 
 	if (disc->flags & FLAG_FREED_BITMAP)
 	{
 		phd->freedSpaceBitmap.extPosition = cpu_to_le32(offset);
 		phd->freedSpaceBitmap.extLength = cpu_to_le32(length);
-		disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - (length / disc->blocksize));
+		value = cpu_to_le32(le32_to_cpu(value) - (length / disc->blocksize));
 	}
 	else if (disc->flags & FLAG_FREED_TABLE)
 	{
@@ -700,19 +810,19 @@ int setup_space(struct udf_disc *disc, struct udf_extent *pspace, uint32_t offse
 		if (disc->flags & FLAG_STRATEGY4096)
 		{
 			phd->freedSpaceTable.extLength = cpu_to_le32(disc->blocksize * 2);
-			disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - 2);
+			value = cpu_to_le32(le32_to_cpu(value) - 2);
 		}
 		else
 		{
 			phd->freedSpaceTable.extLength = cpu_to_le32(disc->blocksize);
-			disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - 1);
+			value = cpu_to_le32(le32_to_cpu(value) - 1);
 		}
 	}
 	else if (disc->flags & FLAG_UNALLOC_BITMAP)
 	{
 		phd->unallocSpaceBitmap.extPosition = cpu_to_le32(offset);
 		phd->unallocSpaceBitmap.extLength = cpu_to_le32(length);
-		disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - (length / disc->blocksize));
+		value = cpu_to_le32(le32_to_cpu(value) - (length / disc->blocksize));
 	}
 	else if (disc->flags & FLAG_UNALLOC_TABLE)
 	{
@@ -720,15 +830,17 @@ int setup_space(struct udf_disc *disc, struct udf_extent *pspace, uint32_t offse
 		if (disc->flags & FLAG_STRATEGY4096)
 		{
 			phd->unallocSpaceTable.extLength = cpu_to_le32(disc->blocksize * 2);
-			disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - 2);
+			value = cpu_to_le32(le32_to_cpu(value) - 2);
 		}
 		else
 		{
 			phd->unallocSpaceTable.extLength = cpu_to_le32(disc->blocksize);
-			disc->udf_lvid->freeSpaceTable[0] = cpu_to_le32(le32_to_cpu(disc->udf_lvid->freeSpaceTable[0]) - 1);
+			value = cpu_to_le32(le32_to_cpu(value) - 1);
 		}
 	}
 
+	memcpy(&disc->udf_lvid->data[sizeof(uint32_t)*0], &value, sizeof(value));
+
 	if (disc->flags & FLAG_SPACE_BITMAP)
 	{
 		struct spaceBitmapDesc *sbd;
@@ -1043,7 +1155,6 @@ void setup_lvd(struct udf_disc *disc, struct udf_extent *mvds, struct udf_extent
 
 	disc->udf_lvd[0]->integritySeqExt.extLength = cpu_to_le32(lvid->blocks * disc->blocksize);
 	disc->udf_lvd[0]->integritySeqExt.extLocation = cpu_to_le32(lvid->start);
-//	((uint16_t *)disc->udf_lvd[0]->domainIdent.identSuffix)[0] = cpu_to_le16(disc->udf_rev);
 
 	desc = set_desc(mvds, TAG_IDENT_LVD, offset, 0, NULL);
 	desc->length = desc->data->length = length;
@@ -1071,12 +1182,6 @@ void setup_pd(struct udf_disc *disc, struct udf_extent *mvds, struct udf_extent
 	}
 	disc->udf_pd[0]->partitionStartingLocation = cpu_to_le32(ext->start);
 	disc->udf_pd[0]->partitionLength = cpu_to_le32(ext->blocks);
-#if 0
-	if (disc->udf_rev >= 0x0200)
-		strcpy(disc->udf_pd[0]->partitionContents.ident, PARTITION_CONTENTS_NSR03);
-	else
-		strcpy(disc->udf_pd[0]->partitionContents.ident, PARTITION_CONTENTS_NSR02);
-#endif
 
 	desc = set_desc(mvds, TAG_IDENT_PD, offset, 0, NULL);
 	desc->length = desc->data->length = length;
@@ -1102,6 +1207,11 @@ void setup_usd(struct udf_disc *disc, struct udf_extent *mvds, struct udf_extent
 	{
 		length += sizeof(extent_ad);
 		disc->udf_usd[0] = realloc(disc->udf_usd[0], length);
+		if (!disc->udf_usd[0])
+		{
+			fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+			exit(1);
+		}
 		disc->udf_usd[0]->numAllocDescs = cpu_to_le32(le32_to_cpu(disc->udf_usd[0]->numAllocDescs)+1);
 		disc->udf_usd[0]->allocDescs[count].extLength = cpu_to_le32(ext->blocks * disc->blocksize);
 		disc->udf_usd[0]->allocDescs[count].extLocation = cpu_to_le32(ext->start);
@@ -1126,8 +1236,6 @@ void setup_iuvd(struct udf_disc *disc, struct udf_extent *mvds, struct udf_exten
 	struct udf_desc *desc;
 	int length = sizeof(struct impUseVolDesc);
 
-//	((uint16_t *)disc->udf_iuvd[0]->impIdent.identSuffix)[0] = cpu_to_le16(disc->udf_rev);
-
 	desc = set_desc(mvds, TAG_IDENT_IUVD, offset, 0, NULL);
 	desc->length = desc->data->length = length;
 	desc->data->buffer = disc->udf_iuvd[0];
@@ -1224,6 +1332,11 @@ void setup_stable(struct udf_disc *disc, struct udf_extent *stable[4], struct ud
 		spm->locSparingTable[i] = cpu_to_le32(stable[i]->start);
 
 	disc->udf_stable[0] = realloc(disc->udf_stable[0], length);
+	if (!disc->udf_stable[0])
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
 	disc->udf_stable[0]->reallocationTableLen = cpu_to_le16(num);
 	for (i=0; i<num; i++)
 	{
@@ -1250,13 +1363,11 @@ void setup_vat(struct udf_disc *disc, struct udf_extent *pspace)
 	struct udf_desc *vtable;
 	struct udf_data *data;
 	uint32_t len, i;
-	struct virtualAllocationTable15 *vat15;
 	struct virtualAllocationTable20 *vat20;
-	struct impUseExtAttr *ea_attr;
+	struct impUseExtAttr *ea;
 	struct LVExtensionEA *ea_lv;
-	uint8_t buffer[(sizeof(*ea_attr)+sizeof(*ea_lv)+3)/4*4];
+	uint8_t buffer[(sizeof(*ea)+sizeof(*ea_lv)+3)/4*4];
 	uint16_t checksum;
-	uint16_t udf_rev_le16;
 	uint32_t min_blocks;
 	uint32_t align;
 
@@ -1305,21 +1416,16 @@ void setup_vat(struct udf_disc *disc, struct udf_extent *pspace)
 	{
 		vtable = udf_create(disc, pspace, (const dchars *)"\x08" UDF_ID_ALLOC, strlen(UDF_ID_ALLOC)+1, offset, NULL, FID_FILE_CHAR_HIDDEN, ICBTAG_FILE_TYPE_UNDEF, 0);
 		disc->vat_entries--; // Remove VAT file itself from VAT table
-		udf_rev_le16 = cpu_to_le16(disc->udf_rev);
 		memset(&buffer, 0, sizeof(buffer));
-		ea_attr = (struct impUseExtAttr *)buffer;
-		ea_attr->attrType = cpu_to_le32(EXTATTR_IMP_USE);
-		ea_attr->attrSubtype = EXTATTR_SUBTYPE;
-		ea_attr->attrLength = cpu_to_le32(sizeof(buffer));
-		ea_attr->impUseLength = cpu_to_le32(sizeof(*ea_lv));
-		ea_attr->impIdent.identSuffix[2] = UDF_OS_CLASS_UNIX;
-		ea_attr->impIdent.identSuffix[3] = UDF_OS_ID_LINUX;
-		memcpy(ea_attr->impIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
-		strcpy((char *)ea_attr->impIdent.ident, UDF_ID_VAT_LVEXTENSION);
-		ea_lv = (struct LVExtensionEA *)&ea_attr->impUse[0];
+		memcpy(&buffer, &default_iuea, sizeof(default_iuea));
+		ea = (struct impUseExtAttr *)buffer;
+		ea->attrLength = cpu_to_le32(sizeof(buffer));
+		ea->impUseLength = cpu_to_le32(sizeof(*ea_lv));
+		strcpy((char *)ea->impIdent.ident, UDF_ID_VAT_LVEXTENSION);
+		ea_lv = (struct LVExtensionEA *)&ea->impUse[0];
 		checksum = 0;
-		for (i = 0; i < sizeof(*ea_attr); ++i)
-			checksum += ((uint8_t *)ea_attr)[i];
+		for (i = 0; i < sizeof(*ea); ++i)
+			checksum += ((uint8_t *)ea)[i];
 		ea_lv->headerChecksum = cpu_to_le16(checksum);
 		if (disc->flags & FLAG_EFE)
 		{
@@ -1339,8 +1445,6 @@ void setup_vat(struct udf_disc *disc, struct udf_extent *pspace)
 		data = alloc_data(disc->vat, disc->vat_entries * sizeof(uint32_t));
 		insert_data(disc, pspace, vtable, data);
 		data = alloc_data(&default_vat15, len);
-		vat15 = data->buffer;
-		memcpy(vat15->vatIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
 		insert_data(disc, pspace, vtable, data);
 	}
 
@@ -1357,6 +1461,12 @@ void add_type1_partition(struct udf_disc *disc, uint16_t partitionNum)
 		sizeof(struct logicalVolDesc) + mtl +
 		sizeof(struct genericPartitionMap1));
 
+	if (!disc->udf_lvd[0])
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
+
 	pm = (struct genericPartitionMap1 *)&disc->udf_lvd[0]->partitionMaps[mtl];
 	mtl += sizeof(struct genericPartitionMap1);
 
@@ -1372,11 +1482,16 @@ void add_type1_partition(struct udf_disc *disc, uint16_t partitionNum)
 		sizeof(struct logicalVolIntegrityDesc) +
 		sizeof(uint32_t) * 2 * (npm + 1) +
 		sizeof(struct logicalVolIntegrityDescImpUse));
-	memmove(&disc->udf_lvid->impUse[sizeof(uint32_t) * 2 * (npm + 1)],
-		&disc->udf_lvid->impUse[sizeof(uint32_t) * 2 * npm],
+	if (!disc->udf_lvid)
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
+	memmove(&disc->udf_lvid->data[sizeof(uint32_t) * 2 * (npm + 1)],
+		&disc->udf_lvid->data[sizeof(uint32_t) * 2 * npm],
 		sizeof(struct logicalVolIntegrityDescImpUse));
-	memmove(&disc->udf_lvid->impUse[sizeof(uint32_t) * (npm + 1)],
-		&disc->udf_lvid->impUse[sizeof(uint32_t) * npm],
+	memmove(&disc->udf_lvid->data[sizeof(uint32_t) * (npm + 1)],
+		&disc->udf_lvid->data[sizeof(uint32_t) * npm],
 		sizeof(uint32_t));
 }
 
@@ -1385,12 +1500,17 @@ void add_type2_sparable_partition(struct udf_disc *disc, uint16_t partitionNum,
 	struct sparablePartitionMap *pm;
 	int mtl = le32_to_cpu(disc->udf_lvd[0]->mapTableLength);
 	int npm = le32_to_cpu(disc->udf_lvd[0]->numPartitionMaps);
-	uint16_t udf_rev_le16 = cpu_to_le16(disc->udf_rev);
 
 	disc->udf_lvd[0] = realloc(disc->udf_lvd[0],
 		sizeof(struct logicalVolDesc) + mtl +
 		sizeof(struct sparablePartitionMap));
 
+	if (!disc->udf_lvd[0])
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
+
 	pm = (struct sparablePartitionMap *)&disc->udf_lvd[0]->partitionMaps[mtl];
 	mtl += sizeof(struct sparablePartitionMap);
 
@@ -1398,7 +1518,6 @@ void add_type2_sparable_partition(struct udf_disc *disc, uint16_t partitionNum,
 	disc->udf_lvd[0]->numPartitionMaps = cpu_to_le32(npm + 1);
 	memcpy(pm, &default_sparmap, sizeof(struct sparablePartitionMap));
 	pm->partitionNum = cpu_to_le16(partitionNum);
-	memcpy(pm->partIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
 	if (packetlen)
 		pm->packetLength = cpu_to_le16(packetlen);
 	pm->numSparingTables = spartable;
@@ -1409,11 +1528,16 @@ void add_type2_sparable_partition(struct udf_disc *disc, uint16_t partitionNum,
 		sizeof(struct logicalVolIntegrityDesc) +
 		sizeof(uint32_t) * 2 * (npm + 1) +
 		sizeof(struct logicalVolIntegrityDescImpUse));
-	memmove(&disc->udf_lvid->impUse[sizeof(uint32_t) * 2 * (npm + 1)],
-		&disc->udf_lvid->impUse[sizeof(uint32_t) * 2 * npm],
+	if (!disc->udf_lvid)
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
+	memmove(&disc->udf_lvid->data[sizeof(uint32_t) * 2 * (npm + 1)],
+		&disc->udf_lvid->data[sizeof(uint32_t) * 2 * npm],
 		sizeof(struct logicalVolIntegrityDescImpUse));
-	memmove(&disc->udf_lvid->impUse[sizeof(uint32_t) * (npm + 1)],
-		&disc->udf_lvid->impUse[sizeof(uint32_t) * npm],
+	memmove(&disc->udf_lvid->data[sizeof(uint32_t) * (npm + 1)],
+		&disc->udf_lvid->data[sizeof(uint32_t) * npm],
 		sizeof(uint32_t));
 }
 
@@ -1449,12 +1573,17 @@ void add_type2_virtual_partition(struct udf_disc *disc, uint16_t partitionNum)
 	struct virtualPartitionMap *pm;
 	int mtl = le32_to_cpu(disc->udf_lvd[0]->mapTableLength);
 	int npm = le32_to_cpu(disc->udf_lvd[0]->numPartitionMaps);
-	uint16_t udf_rev_le16 = cpu_to_le16(disc->udf_rev);
 
 	disc->udf_lvd[0] = realloc(disc->udf_lvd[0],
 		sizeof(struct logicalVolDesc) + mtl +
 		sizeof(struct virtualPartitionMap));
 
+	if (!disc->udf_lvd[0])
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
+
 	pm = (struct virtualPartitionMap *)&disc->udf_lvd[0]->partitionMaps[mtl];
 	mtl += sizeof(struct virtualPartitionMap);
 
@@ -1462,18 +1591,22 @@ void add_type2_virtual_partition(struct udf_disc *disc, uint16_t partitionNum)
 	disc->udf_lvd[0]->numPartitionMaps = cpu_to_le32(npm + 1);
 	memcpy(pm, &default_virtmap, sizeof(struct virtualPartitionMap));
 	pm->partitionNum = cpu_to_le16(partitionNum);
-	memcpy(pm->partIdent.identSuffix, &udf_rev_le16, sizeof(udf_rev_le16));
 
 	disc->udf_lvid->numOfPartitions = cpu_to_le32(npm + 1);
 	disc->udf_lvid = realloc(disc->udf_lvid,
 		sizeof(struct logicalVolIntegrityDesc) +
 		sizeof(uint32_t) * 2 * (npm + 1) +
 		sizeof(struct logicalVolIntegrityDescImpUse));
-	memmove(&disc->udf_lvid->impUse[sizeof(uint32_t) * 2 * (npm + 1)],
-		&disc->udf_lvid->impUse[sizeof(uint32_t) * 2 * npm],
+	if (!disc->udf_lvid)
+	{
+		fprintf(stderr, "%s: Error: realloc failed: %s\n", appname, strerror(errno));
+		exit(1);
+	}
+	memmove(&disc->udf_lvid->data[sizeof(uint32_t) * 2 * (npm + 1)],
+		&disc->udf_lvid->data[sizeof(uint32_t) * 2 * npm],
 		sizeof(struct logicalVolIntegrityDescImpUse));
-	memmove(&disc->udf_lvid->impUse[sizeof(uint32_t) * (npm + 1)],
-		&disc->udf_lvid->impUse[sizeof(uint32_t) * npm],
+	memmove(&disc->udf_lvid->data[sizeof(uint32_t) * (npm + 1)],
+		&disc->udf_lvid->data[sizeof(uint32_t) * npm],
 		sizeof(uint32_t));
 }
 
