@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (C) 2017-2019  Pali Rohár <pali.rohar@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <errno.h>
+#include <inttypes.h>
 #include <locale.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -155,7 +156,7 @@ static void dump_space(struct udf_disc *disc)
 				continue;
 			if (start_ext->space_type & (USPACE|RESERVED))
 				continue;
-			printf("start=%lu, blocks=%lu, type=%s\n", (unsigned long int)start_ext->start, (unsigned long int)start_ext->blocks, udf_space_type_str[i]);
+			printf("start=%"PRIu32", blocks=%"PRIu32", type=%s\n", start_ext->start, start_ext->blocks, udf_space_type_str[i]);
 		}
 	}
 }
@@ -163,17 +164,22 @@ static void dump_space(struct udf_disc *disc)
 int main(int argc, char *argv[])
 {
 	struct udf_disc disc;
+	struct stat stat;
 	dstring vsid[128];
 	char uuid[17];
 	char *filename;
 	struct logicalVolDesc *lvd;
 	struct primaryVolDesc *pvd;
 	struct partitionDesc *pd;
+	struct domainIdentSuffix *dis;
 	uint32_t serial_num;
 	uint32_t used_blocks;
 	uint32_t behind_blocks;
+	int soft_write_protect;
+	int hard_write_protect;
 	size_t ret;
 	int fd;
+	int fd2;
 
 	setlocale(LC_CTYPE, "");
 	appname = "udfinfo";
@@ -200,6 +206,15 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	if (fstat(fd, &stat) == 0 && S_ISBLK(stat.st_mode))
+	{
+		fd2 = open(filename, O_RDONLY|O_EXCL);
+		if (fd2 >= 0)
+			close(fd2);
+		else if (errno == EBUSY)
+			fprintf(stderr, "%s: Warning: Device '%s' is busy, %s may report bogus information\n", appname, filename, appname);
+	}
+
 	disc.blksize = get_size(fd);
 	disc.blkssz = get_sector_size(fd);
 
@@ -281,19 +296,19 @@ int main(int argc, char *argv[])
 	print_dstring(&disc, "vsid", vsid, sizeof(vsid));
 	print_dstring(&disc, "fsid", disc.udf_fsd ? disc.udf_fsd->fileSetIdent : NULL, sizeof(disc.udf_fsd->fileSetIdent));
 	print_dstring(&disc, "fullvsid", pvd ? pvd->volSetIdent : NULL, sizeof(pvd->volSetIdent));
-	printf("winserialnum=0x%08lx\n", (unsigned long int)serial_num);
-	printf("blocksize=%d\n", disc.blocksize);
-	printf("blocks=%lu\n", (unsigned long int)disc.blocks);
-	printf("usedblocks=%lu\n", (unsigned long int)used_blocks);
-	printf("freeblocks=%lu\n", (unsigned long int)disc.free_space_blocks);
-	printf("behindblocks=%lu\n", (unsigned long int)behind_blocks);
-	printf("numfiles=%lu\n", (unsigned long int)disc.num_files);
-	printf("numdirs=%lu\n", (unsigned long int)disc.num_dirs);
-	printf("udfrev=%x.%02x\n", (unsigned int)(disc.udf_rev >> 8), (unsigned int)(disc.udf_rev & 0xFF));
-	printf("udfwriterev=%x.%02x\n", (unsigned int)(disc.udf_write_rev >> 8), (unsigned int)(disc.udf_write_rev & 0xFF));
+	printf("winserialnum=0x%08"PRIx32"\n", serial_num);
+	printf("blocksize=%"PRIu32"\n", disc.blocksize);
+	printf("blocks=%"PRIu32"\n", disc.blocks);
+	printf("usedblocks=%"PRIu32"\n", used_blocks);
+	printf("freeblocks=%"PRIu32"\n", disc.free_space_blocks);
+	printf("behindblocks=%"PRIu32"\n", behind_blocks);
+	printf("numfiles=%"PRIu32"\n", disc.num_files);
+	printf("numdirs=%"PRIu32"\n", disc.num_dirs);
+	printf("udfrev=%"PRIx16".%02"PRIx16"\n", disc.udf_rev >> 8, disc.udf_rev & 0xFF);
+	printf("udfwriterev=%"PRIx16".%02"PRIx16"\n", disc.udf_write_rev >> 8, disc.udf_write_rev & 0xFF);
 
 	if (disc.vat_block)
-		printf("vatblock=%lu\n", (unsigned long int)disc.vat_block);
+		printf("vatblock=%"PRIu32"\n", disc.vat_block);
 
 	if (disc.udf_lvid)
 	{
@@ -340,6 +355,29 @@ int main(int argc, char *argv[])
 	else
 		printf("accesstype=unknown\n");
 
+	soft_write_protect = hard_write_protect = 0;
+
+	if (disc.udf_fsd)
+	{
+		dis = (struct domainIdentSuffix *)disc.udf_fsd->domainIdent.identSuffix;
+		if (dis->domainFlags & DOMAIN_FLAGS_HARD_WRITE_PROTECT)
+			soft_write_protect = hard_write_protect = 1;
+		else if (dis->domainFlags & DOMAIN_FLAGS_SOFT_WRITE_PROTECT)
+			soft_write_protect = 1;
+	}
+
+	if (lvd)
+	{
+		dis = (struct domainIdentSuffix *)lvd->domainIdent.identSuffix;
+		if (dis->domainFlags & DOMAIN_FLAGS_HARD_WRITE_PROTECT)
+			soft_write_protect = hard_write_protect = 1;
+		else if (dis->domainFlags & DOMAIN_FLAGS_SOFT_WRITE_PROTECT)
+			soft_write_protect = 1;
+	}
+
+	printf("softwriteprotect=%s\n", soft_write_protect ? "yes" : "no");
+	printf("hardwriteprotect=%s\n", hard_write_protect ? "yes" : "no");
+
 	dump_space(&disc);
 
 	return 0;
