@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (C) 2017-2019  Pali Rohár <pali.rohar@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <errno.h>
+#include <inttypes.h>
 #include <locale.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -139,7 +140,7 @@ static void write_desc(int fd, struct udf_disc *disc, enum udf_space_type type,
 			if (!desc->data || desc->data->buffer != buffer)
 				continue;
 
-			printf("  ... at block %lu\n", (unsigned long int)(ext->start + desc->offset));
+			printf("  ... at block %"PRIu32"\n", ext->start + desc->offset);
 
 			offset = (off_t)disc->blocksize * (ext->start + desc->offset);
 			off = lseek(fd, offset, SEEK_SET);
@@ -156,7 +157,7 @@ static void write_desc(int fd, struct udf_disc *disc, enum udf_space_type type,
 
 			if (!(disc->flags & FLAG_NO_WRITE))
 			{
-				ret = write(fd, desc->data->buffer, desc->data->length);
+				ret = write_nointr(fd, desc->data->buffer, desc->data->length);
 				if (ret >= 0 && (size_t)ret != desc->data->length)
 				{
 					errno = EIO;
@@ -185,8 +186,10 @@ int main(int argc, char *argv[])
 	struct partitionDesc *pd;
 	struct logicalVolDesc *lvd;
 	struct impUseVolDescImpUse *iuvdiu;
+	struct domainIdentSuffix *dis;
 	size_t len;
 	int fd;
+	int fd2;
 	int i;
 	char buf[256];
 	dstring new_lvid[128];
@@ -269,19 +272,18 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	if (fstat(fd, &stat) != 0)
+	{
+		fprintf(stderr, "%s: Error: Cannot stat device '%s': %s\n", appname, filename, strerror(errno));
+		exit(1);
+	}
+
 	if (update)
 	{
-		int fd2;
 		int flags2;
 		char filename2[64];
 		const char *error;
 
-		if (fstat(fd, &stat) != 0)
-		{
-			fprintf(stderr, "%s: Error: Cannot stat device '%s': %s\n", appname, filename, strerror(errno));
-			exit(1);
-		}
-
 		if (!(disc.flags & FLAG_NO_WRITE))
 			flags2 = O_RDWR;
 		else
@@ -300,15 +302,9 @@ int main(int argc, char *argv[])
 		fd2 = open(filename2, flags2);
 		if (fd2 < 0)
 		{
-			if (errno != ENOENT)
-			{
-				error = (errno != EBUSY) ? strerror(errno) : "Device is busy, maybe mounted?";
-				fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, error);
-				exit(1);
-			}
-
 			// Fallback to orignal filename when /proc is not available, but this introduce race condition between stat and open
-			fd2 = open(filename, flags2);
+			if (errno == ENOENT)
+				fd2 = open(filename, flags2);
 			if (fd2 < 0)
 			{
 				error = (errno != EBUSY) ? strerror(errno) : "Device is busy, maybe mounted?";
@@ -320,6 +316,14 @@ int main(int argc, char *argv[])
 		close(fd);
 		fd = fd2;
 	}
+	else if (S_ISBLK(stat.st_mode))
+	{
+		fd2 = open(filename, O_RDONLY|O_EXCL);
+		if (fd2 >= 0)
+			close(fd2);
+		else if (errno == EBUSY)
+			fprintf(stderr, "%s: Warning: Device '%s' is busy, %s may report bogus information\n", appname, filename, appname);
+	}
 
 	disc.blksize = get_size(fd);
 	disc.blkssz = get_sector_size(fd);
@@ -366,7 +370,7 @@ int main(int argc, char *argv[])
 
 	if (disc.udf_write_rev > 0x0260)
 	{
-		fprintf(stderr, "%s: Error: Minimal UDF Write Revision is %x.%02x, but udflabel supports only 2.60\n", appname, (unsigned int)(disc.udf_write_rev >> 8), (unsigned int)(disc.udf_write_rev & 0xFF));
+		fprintf(stderr, "%s: Error: Minimal UDF Write Revision is %"PRIx16".%02"PRIx16", but udflabel supports only 2.60\n", appname, disc.udf_write_rev >> 8, disc.udf_write_rev & 0xFF);
 		exit(1);
 	}
 
@@ -436,6 +440,40 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	if (disc.udf_lvd[0] && update_lvd)
+	{
+		dis = (struct domainIdentSuffix *)disc.udf_lvd[0]->domainIdent.identSuffix;
+		if (dis->domainFlags & (DOMAIN_FLAGS_SOFT_WRITE_PROTECT|DOMAIN_FLAGS_HARD_WRITE_PROTECT))
+		{
+			if (!force)
+			{
+				fprintf(stderr, "%s: Error: Cannot overwrite write protected Logical Volume Descriptor\n", appname);
+				exit(1);
+			}
+			else
+			{
+				fprintf(stderr, "%s: Warning: Trying to overwrite write protected Logical Volume Descriptor\n", appname);
+			}
+		}
+	}
+
+	if (disc.udf_fsd && update_fsd)
+	{
+		dis = (struct domainIdentSuffix *)disc.udf_fsd->domainIdent.identSuffix;
+		if (dis->domainFlags & (DOMAIN_FLAGS_SOFT_WRITE_PROTECT|DOMAIN_FLAGS_HARD_WRITE_PROTECT))
+		{
+			if (!force)
+			{
+				fprintf(stderr, "%s: Error: Cannot overwrite write protected File Set Descriptor\n", appname);
+				exit(1);
+			}
+			else
+			{
+				fprintf(stderr, "%s: Warning: Trying to overwrite write protected File Set Descriptor\n", appname);
+			}
+		}
+	}
+
 	if (update_pvd)
 	{
 		if (!disc.udf_pvd[0] || !check_desc(disc.udf_pvd[0], sizeof(*disc.udf_pvd[0])))
@@ -636,7 +674,7 @@ int main(int argc, char *argv[])
 		{
 			if (fdatasync(fd) != 0)
 			{
-				fprintf(stderr, "%s: Synchronization failed: %s\n", appname, strerror(errno));
+				fprintf(stderr, "%s: Error: Synchronization to device '%s' failed: %s\n", appname, filename, strerror(errno));
 				exit(1);
 			}
 		}
@@ -673,13 +711,19 @@ int main(int argc, char *argv[])
 	printf("Synchronizing...\n");
 	if (!(disc.flags & FLAG_NO_WRITE))
 	{
-		if (fdatasync(fd) != 0)
+		if (fsync(fd) != 0)
 		{
-			fprintf(stderr, "%s: Synchronization failed: %s\n", appname, strerror(errno));
+			fprintf(stderr, "%s: Error: Synchronization to device '%s' failed: %s\n", appname, filename, strerror(errno));
 			exit(1);
 		}
 	}
 
+	if (close(fd) != 0 && errno != EINTR)
+	{
+		fprintf(stderr, "%s: Error: Closing device '%s' failed: %s\n", appname, filename, strerror(errno));
+		exit(1);
+	}
+
 	printf("Done\n");
 	return 0;
 }
