@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (C) 2017-2019  Pali Rohár <pali.rohar@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <inttypes.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -83,8 +84,8 @@ static void usage(void)
 		"\n"
 		"Encoding Options:\n"
 		"\t--locale           Identifier options are encoded according to current locale (default)\n"
-		"\t--u8               Identifier options are encoded in 8-bit OSTA Compressed Unicode format\n"
-		"\t--u16              Identifier options are encoded in 16-bit OSTA Compressed Unicode format\n"
+		"\t--u8               Identifier options are encoded in Latin1\n"
+		"\t--u16              Identifier options are encoded in UTF-16BE\n"
 		"\t--utf8             Identifier options are encoded in UTF-8\n"
 	);
 	exit(1);
@@ -103,7 +104,7 @@ static void process_uuid_arg(const char *arg, char *new_uuid)
 			uuid_time = cur_time & 0xFFFFFFFF;
 		else
 			uuid_time = randu32();
-		snprintf(new_uuid, 17, "%08lx%08lx", (unsigned long int)uuid_time, (unsigned long int)randu32());
+		snprintf(new_uuid, 17, "%08"PRIx32"%08"PRIx32"", uuid_time, randu32());
 		return;
 	}
 
