@@ -74,18 +74,22 @@ int	writePacket(struct packetbuf* pb);
 void markBlock(enum markAction action, uint32_t blkno) {
     uint8_t	*bm;
     uint8_t	mask;
+    uint32_t	value;
 
     spaceMapDirty = 1;
     bm = spaceMap->bitmap + (blkno >> 3);
     mask = 1 << (blkno & 7);
+    memcpy(&value, &lvid->data[sizeof(uint32_t)*pd->partitionNumber], sizeof(value));
 
     if( action == FREE ) {
 	*bm |= mask;
-	lvid->freeSpaceTable[pd->partitionNumber]++;
+	value++;
     } else {
 	*bm &= ~mask;
-	lvid->freeSpaceTable[pd->partitionNumber]--;
+	value--;
     }
+
+    memcpy(&lvid->data[sizeof(uint32_t)*pd->partitionNumber], &value, sizeof(value));
 }
 
 /*	GetExtents()
@@ -95,10 +99,12 @@ void markBlock(enum markAction action, uint32_t blkno) {
  *
  *	Return value: lenAllocDescs if extents found, 0 if not.
  */
+typedef uint32_t __attribute__((may_alias)) uint32_t_a;
 int getExtents(uint32_t requestedLength, short_ad *extents) {
     uint32_t	blkno, lengthFound = 0;
     short_ad	*ext;
-    uint32_t	mask, *bm;
+    uint32_t	mask;
+    uint32_t_a	*bm;
 
     if( medium == CDR ) {
 	/* check space availability */
@@ -115,7 +121,7 @@ int getExtents(uint32_t requestedLength, short_ad *extents) {
     // clear extents
     ext = extents;
     ext->extLength = 0;
-    bm = ((uint32_t*)spaceMap->bitmap) - 1;
+    bm = ((uint32_t_a*)spaceMap->bitmap) - 1;
     mask = 0;
     blkno = 0xFFFFFFFF;
 
@@ -396,10 +402,10 @@ void updateSparingTable() {
 	} else { // DISK_IMAGE
 	    off = lseek(device, 2048 * pbn, SEEK_SET);
 	    if( off == (off_t)-1 )
-		fail("writeSparingTable at %d: %m\n", pbn);
+		fail("writeSparingTable at %d: %s\n", pbn, strerror(errno));
 	    len = write(device, pb->pkt, 32 * 2048);
 	    if( len < 0 )
-		fail("writeSparingTable at %d: %m\n", pbn);
+		fail("writeSparingTable at %d: %s\n", pbn, strerror(errno));
 	    if( len != 32 * 2048 )
 		fail("writeSparingTable at %d: %s\n", pbn, strerror(EIO));
 	}
@@ -424,10 +430,10 @@ readPacket(struct packetbuf* pb)
     } else {
 	off = lseek(device, 2048 * physical, SEEK_SET);
 	if( off == (off_t)-1 )
-	    fail("readPacket: lseek failed %m\n");
+	    fail("readPacket: lseek failed %s\n", strerror(errno));
 	len = read(device, pb->pkt, 32 * 2048);
 	if( len < 0 )
-	    fail("readPacket: read failed %m\n");
+	    fail("readPacket: read failed %s\n", strerror(errno));
 	if( len != 32 * 2048 )
 	    fail("readPacket: read failed %s\n", strerror(EIO));
 	ret = 0;
@@ -479,10 +485,10 @@ writePacket(struct packetbuf* pb)
 #endif
 	off = lseek(device, 2048 * physical, SEEK_SET);
 	if( off == (off_t)-1 )
-	    fail("writePacket: writeHD failed %m\n");
+	    fail("writePacket: writeHD failed %s\n", strerror(errno));
 	len = write(device, pb->pkt, 32 * 2048);
 	if( len < 0 )
-	    fail("writePacket: writeHD failed %m\n");
+	    fail("writePacket: writeHD failed %s\n", strerror(errno));
 	if( len != 32 * 2048 )
 	    fail("writePacket: writeHD failed %s\n", strerror(EIO));
 	ret = 0;
@@ -801,10 +807,10 @@ initIO(char *filename)
 	/* heuristically determine medium imitated on disk image based on VAT FileEntry in block 512 */
 	off = lseek(device, 2048 * 512, SEEK_SET);
 	if( off == (off_t)-1 )
-	    fail("initIO: lseek %s failed: %m\n", filename);
+	    fail("initIO: lseek %s failed: %s\n", filename, strerror(errno));
 	len = read(device, &ident, 2);
 	if( len < 0 )
-	    fail("initIO: read %s failed: %m\n", filename);
+	    fail("initIO: read %s failed: %s\n", filename, strerror(errno));
 	if( len != 2 )
 	    fail("initIO: read %s failed: %s\n", filename, strerror(EIO));
 	medium = ident == TAG_IDENT_VDP ? CDR : CDRW;
